/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIESREGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2018 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/*******************************************************************************
* File Name    : r_iic_lib.c
* Version      : 2.00
* Device(s)    : R5F100LE
* Tool-Chain   : CCRL
* Description  : This file implements IIC control functions.
* Creation Date: 2015/12/26
* Modified Date: 2018/09/30
*******************************************************************************/

/*******************************************************************************
Includes
*******************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"
#include "r_iic_lib.h"
#include "r_timer_user.h"

/*******************************************************************************
Pragma directive
*******************************************************************************/
#pragma interrupt r_iica0_interrupt(vect=INTIICA0,bank=RB1)

/*******************************************************************************
Global variables
*******************************************************************************/

#define     WAITTIME        ( 1000 )    /* limit of start condition     */
#define     ON_COMMU        ( 0x01 )    /* on communication (transmit)  */
#define     DUMMY_DATA      ( 0xFF )    /* dummy data for receive       */


volatile uint8_t   g_iica0_status;      /* iica0 master operationflag   */
/*  0x00 : success(IIC_SUCCESS)
    0x01 : transmit operating(IIC_USING) same as ON_COMMU
    0x84 : no ACK from slave for data(NO_ACK)
    0x88 : no ACK for slave address(NO_SLAVE)
    0x8C : I2C BUS busy(BUS_BUSY)
*/
volatile uint8_t * gp_iica0_rx_address; /* iica0 receive buffer address */
volatile uint16_t  g_iica0_rx_len;      /* iica0 receive data length    */
volatile uint16_t  g_iica0_rx_cnt;      /* iica0 receive data count     */
volatile uint8_t * gp_iica0_tx_address; /* iica0 send buffer address    */
volatile uint16_t  g_iica0_tx_cnt;      /* iica0 send data count        */

/*******************************************************************************
Local functions
*******************************************************************************/

uint8_t R_IICA0_bus_check(void);        /* check I2C bus status         */

/*******************************************************************************
* Function Name: R_IIC_Master_Send
* Description  : This function starts iica0 in transmision mode.
* Arguments    : adr -
*                    slave address
*                tx_buf -
*                    transfer data pointer
*                tx_num -
*                    transmit data number
* Return Value : none
*                    IIC_SUCCESS: get I2C bus and transmit slave address
*                    BUS_BUSY: I2C bus is busy
********************************************************************************/
uint8_t R_IIC_Master_Send(uint8_t adr, uint8_t * const tx_buf, uint16_t tx_num)
{

    uint8_t status;

    status = R_IICA0_bus_check();       /* check I2C bus status         */

    if ( IIC_SUCCESS == status )
    {
/*--------------------------------------------------------------*/
/*      get I2C bus and start to access I2C bus                 */
/*--------------------------------------------------------------*/

        g_iica0_tx_cnt = tx_num;        /* set data number to transmit  */
        gp_iica0_tx_address = tx_buf;   /* set data pointer             */

/*--------------------------------------------------------------*/
/*      get I2C bus and start to access I2C bus                 */
/*--------------------------------------------------------------*/

        IICA0 = adr & 0xFE;             /* transmit slave address       */

    }
    else
    {
        /*  do nothing  */
    }

    return (status);

}

/*******************************************************************************
* Function Name: R_IIC_Master_Receive
* Description  : This function starts iica0 in receive mode.
* Arguments    : adr -
*                    slave address
*                rx_buf -
*                    received data buffer pointer
*                rx_num -
*                    receive data size+1
* Return Value : status -
*                    IIC_SUCCESS: get I2C bus and transmit slave address
*                    BUS_BUSY: I2C bus is busy
********************************************************************************/
uint8_t R_IIC_Master_Receive(uint8_t adr, uint8_t * const rx_buf, uint16_t rx_num)
{

    MD_STATUS status;

    status = R_IICA0_bus_check();       /* check I2C bus status         */

    if ( IIC_SUCCESS == status )
    {

/*--------------------------------------------------------------*/
/*      get I2C bus and set access parameters                   */
/*--------------------------------------------------------------*/

        g_iica0_rx_len = rx_num;        /* set data number to transmit  */
        g_iica0_rx_cnt = 0U;
        gp_iica0_rx_address = rx_buf;   /* set data pointer             */

/*--------------------------------------------------------------*/
/*      start to access I2C bus                                 */
/*--------------------------------------------------------------*/

        IICA0 = adr | 0x01;             /* transmit slave address       */

    }

    return (status);
}

/*******************************************************************************
* Function Name: R_IIC_check_comstate
* Description  : This function get communication status
* Arguments    : none
* Return Value : result status
                0x00 : success(IIC_SUCCESS)
                0x01 : transmit operating(IIC_USING)
                0x82 : BUS busy
                0x84 : no ACK from slave for data(NO_ACK)
                0x8C : no ACK for slave address(NO_SLAVE)
********************************************************************************/
uint8_t R_IIC_check_comstate(void)
{

    return( g_iica0_status );

}

/*******************************************************************************
* Function Name: R_IIC_wait_comend
* Description  : This function wait for communication end
* Arguments    : none
* Return Value : result status
                0x00 : success
                other : error and issue STOP conditiopn
********************************************************************************/
uint8_t R_IIC_wait_comend(void)
{
    uint8_t status;

/*--------------------------------------------------------------*/
/*      wait for communication end by checking g_iica0_status   */
/*--------------------------------------------------------------*/

    while( IIC_USING == ( g_iica0_status & STS_MASK ) )
    {
        HALT();                         /* wait for communication end   */
    }

    status = g_iica0_status;            /* set result status            */

    if ( IIC_SUCCESS != status )
    {

/*--------------------------------------------------------------*/
/*      communication error then issue STOP conditiopn          */
/*--------------------------------------------------------------*/

        R_IIC_StopCondition();          /* issue STOP conditiopn        */
        delay_us(60);                   /* wait for 60us                */
    }

    return( status );
}

/*******************************************************************************
* Function Name: R_IIC_StopCondition
* Description  : This function issue stop condition
* Arguments    : none
* Return Value : result
                0x00 : success
                other : error to detect STOP condition
********************************************************************************/
uint8_t R_IIC_StopCondition(void)
{

    uint8_t status;
    uint16_t wait;

    status = IIC_SUCCESS;               /* set dummy status "success"   */

/*--------------------------------------------------------------*/
/*      Issue STOP conditiopn to stop using I2C bus             */
/*--------------------------------------------------------------*/

    SPT0 = TRUTH;                       /* issue stop condition to stop */

/*--------------------------------------------------------------*/
/*      Wait for detecting STOP condition                       */
/*--------------------------------------------------------------*/

    for( wait = WAITTIME ; ( wait > 0 ) && ( FALSE == SPD0 ) ; wait-- )
    {
        NOP();                          /* wait for detection           */
    }

    if ( FALSE == SPD0 )
    {
       status = BUS_BUSY;              /* not detect stop condition    */
    }

    return (status);
}

/*******************************************************************************
* Function Name: R_IICA0_bus_check
* Description  : This function check I2C bus and issue start condition
* Arguments    : none
* Return Value : status -
*                 IIC_SUCCESS: successfuly issue start condition and get I2C bus
*                 BUS_BUSY: I2C bus is busy
********************************************************************************/
uint8_t R_IICA0_bus_check(void)
{
    uint8_t status;
    uint16_t wait;

/*--------------------------------------------------------------*/
/*      Check I2C bus is used or in waiting state               */
/*--------------------------------------------------------------*/

    status = BUS_BUSY;                  /* set dummy error flag         */

    if ( ( FALSE == IICBSY0 ) || ( TRUTH == MSTS0 ))    /* Check I2C bus */
    {

/*--------------------------------------------------------------*/
/*      I2C bus is usable, then issue start condition           */
/*--------------------------------------------------------------*/

        STT0 = TRUTH;                   /* issue IICA0 start condition  */

/*--------------------------------------------------------------*/
/*      Wait for start condition is detected                    */
/*--------------------------------------------------------------*/

        for( wait = WAITTIME ; ( wait > 0 )&&( FALSE == STD0 ) ; wait--)
        {
            NOP();
        }

        if ( ( TRUTH == MSTS0 ) && ( TRUTH == STD0 ))
        {
/*--------------------------------------------------------------*/
/*      success to get I2C bus                                  */
/*--------------------------------------------------------------*/

            status = IIC_SUCCESS;       /* set status OK                */
            g_iica0_status = ON_COMMU;  /* set communication flag       */
        }

    }

    return (status);
}

/********************************************************************************
* Function Name: r_iica0_interrupt
* Description  : This function is INTIICA0 interrupt service routine.
* Arguments    : None
* Return Value : None
*********************************************************************************/
static void r_iica0_interrupt(void)
{

    if ( TRUTH == MSTS0 )               /* check slave mode operation ? */
    {

/*==============================================================*/
/*      master mode interrupt operation start                   */
/*==============================================================*/

        if ( TRUTH == STD0 )            /* slave address transmit end   */
        {
            if ( TRUTH == ACKD0 )
            {

/*--------------------------------------------------------------*/
/*      ACK response and start data transfe                     */
/*--------------------------------------------------------------*/

                if ( TRUTH == TRC0 )

/*==============================================================*/
/*	    data transmit mode operation	                        */
/*==============================================================*/

                {
                    if ( g_iica0_tx_cnt > 0 )
                    {

/*--------------------------------------------------------------*/
/*      there is more data to transmit then transmit it         */
/*--------------------------------------------------------------*/

                        IICA0 = *gp_iica0_tx_address;   /* set transmit data        */
                        gp_iica0_tx_address++;          /* move pointer to next data*/
                        g_iica0_tx_cnt--;               /* count down data number   */
                    }
                    else
                    {

/*--------------------------------------------------------------*/
/*      whole data has been transmitted                         */
/*--------------------------------------------------------------*/

                        g_iica0_status = IIC_SUCCESS;   /* set data transfer end    */
                    }
                }
                else
                {

/*==============================================================*/
/*	    data receive mode operation	                            */
/*==============================================================*/

                    WTIM0 = FALSE;              /* set 8bit wait mode           */
                    ACKE0 = TRUTH;
                    
                    IICA0 = DUMMY_DATA;         /* canceling wait and start Rx  */
                }
            }
            else
            {

/*--------------------------------------------------------------*/
/*      there is no slave that respond ACK for slave address    */
/*--------------------------------------------------------------*/

                g_iica0_status = NO_SLAVE;      /* set communication error flag */
            }

        }
        else
        {

/*==============================================================*/
/*      data transfer stage                                     */
/*==============================================================*/

            if ( TRUTH == TRC0 )
            {

/*==============================================================*/
/*      transmit operation mode                                 */
/*==============================================================*/

                if ( TRUTH == ACKD0 )
                {

/*--------------------------------------------------------------*/
/*      success data transmit and check remaining data          */
/*--------------------------------------------------------------*/

                    if ( g_iica0_tx_cnt > 0 )   /* check transmit data number   */
                    {

/*--------------------------------------------------------------*/
/*      change prepare to index next transmit data              */
/*--------------------------------------------------------------*/

                        IICA0 = *gp_iica0_tx_address;   /* transmit data            */
                        gp_iica0_tx_address++;          /* move pointer to next data*/
                        g_iica0_tx_cnt--;               /* count down data number   */
                    }
                    else
                    {

/*--------------------------------------------------------------*/
/*      data transmit complete, set end flag and exit           */
/*--------------------------------------------------------------*/

                        g_iica0_status = IIC_SUCCESS;   /* set data transfer end    */

                    }
                }
                else
                {

/*--------------------------------------------------------------*/
/*      NACK error                                              */
/*--------------------------------------------------------------*/

                    g_iica0_status = NO_ACK;            /* set communication error flag */
                }
            }
            else
            {

/*==============================================================*/
/*      receive operation mode                                  */
/*==============================================================*/

                if ( g_iica0_rx_cnt < g_iica0_rx_len )
                {

/*--------------------------------------------------------------*/
/*      store recieved data                                     */
/*--------------------------------------------------------------*/

                    *gp_iica0_rx_address = IICA0;   /* store received data  */
                    gp_iica0_rx_address++;          /* move store pointer   */
                    g_iica0_rx_cnt++;               /* countup data number  */
                    if ( g_iica0_rx_cnt == g_iica0_rx_len )
                    {

/*--------------------------------------------------------------*/
/*      receive last data operation (send NACK and change to    */
/*      9 clock wait mode                                       */
/*--------------------------------------------------------------*/

                        ACKE0 = FALSE;              /* NACK for last data   */
                        WTIM0 = TRUTH;              /* change to 9clock wait*/
                        WREL0 = TRUTH;              /* canceling wait to end*/
                    }
                    else
                    {
                        WREL0 = TRUTH;              /* start next reception */
                    }
                }
                else
                {

/*--------------------------------------------------------------*/
/*      receive operation complete at 9th clock                 */
/*--------------------------------------------------------------*/

                    g_iica0_status = IIC_SUCCESS;   /* set data receive end */


                }

            }

        }

/*==============================================================*/
/*      master transfer operation end                           */
/*==============================================================*/

    }

}


